@@ -518,7 +518,7 @@ djvu_document_class_init (DjvuDocumentClass *klass)
 
 static gchar *
 djvu_text_copy (DjvuDocument *djvu_document,
-		gint           page,
+		gint           page_num,
 		EvRectangle  *rectangle)
 {
 	miniexp_t page_text;
@@ -526,7 +526,7 @@ djvu_text_copy (DjvuDocument *djvu_document,
 
 	while ((page_text =
 		ddjvu_document_get_pagetext (djvu_document->d_document,
-					     page, "char")) == miniexp_dummy)
+					     page_num, "char")) == miniexp_dummy)
 		djvu_handle_events (djvu_document, TRUE, NULL);
 
 	if (page_text != miniexp_nil) {
@@ -734,6 +734,7 @@ djvu_document_file_exporter_end (EvFileExporter *exporter)
 {
 	int d_optc = 1; 
 	const char *d_optv[d_optc];
+	ddjvu_job_t *job;
 
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (exporter);
 
@@ -745,7 +746,7 @@ djvu_document_file_exporter_end (EvFileExporter *exporter)
 	
 	d_optv[0] = djvu_document->opts->str; 
 
-	ddjvu_job_t * job = ddjvu_document_print(djvu_document->d_document, fn, d_optc, d_optv);
+	job = ddjvu_document_print(djvu_document->d_document, fn, d_optc, d_optv);
 	while (!ddjvu_job_done(job)) {	
 		djvu_handle_events (djvu_document, TRUE, NULL);
 	}
