@@ -55,7 +55,7 @@ static const gchar *const ev_view_accessible_action_descriptions[] =
 };
 
 struct _EvViewAccessiblePrivate {
-	guint         current_page;
+	EvDocumentModel *model;
 
 	/* AtkAction */
 	gchar        *action_descriptions[LAST_ACTION];
@@ -75,20 +75,31 @@ G_DEFINE_TYPE_WITH_CODE (EvViewAccessible, ev_view_accessible, GTK_TYPE_CONTAINE
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_HYPERTEXT, ev_view_accessible_hypertext_iface_init)
 	)
 
+static void
+clear_cache (EvViewAccessible *accessible)
+{
+	EvViewAccessiblePrivate* priv = accessible->priv;
+
+	g_clear_object (&priv->buffer);
+	g_clear_pointer (&priv->links, (GDestroyNotify)g_hash_table_destroy);
+}
+
 static void
 ev_view_accessible_finalize (GObject *object)
 {
 	EvViewAccessiblePrivate *priv = EV_VIEW_ACCESSIBLE (object)->priv;
 	int i;
 
+	if (priv->model) {
+		g_signal_handlers_disconnect_by_data (priv->model, object);
+		g_object_unref (priv->model);
+		priv->model = NULL;
+	}
 	if (priv->action_idle_handler)
 		g_source_remove (priv->action_idle_handler);
 	for (i = 0; i < LAST_ACTION; i++)
 		g_free (priv->action_descriptions [i]);
-	if (priv->buffer)
-		g_object_unref (priv->buffer);
-	if (priv->links)
-		g_hash_table_destroy (priv->links);
+	clear_cache (EV_VIEW_ACCESSIBLE (object));
 
 	G_OBJECT_CLASS (ev_view_accessible_parent_class)->finalize (object);
 }
@@ -131,21 +142,16 @@ ev_view_accessible_get_text_buffer (EvViewAccessible *accessible, EvView *view)
 	const gchar *retval = NULL;
 	EvViewAccessiblePrivate* priv = accessible->priv;
 
+	if (priv->buffer) {
+		return priv->buffer;
+	}
+
 	page_cache = view->page_cache;
 	if (!page_cache) {
 		return NULL;
 	}
 
-	if (view->current_page == priv->current_page && priv->buffer) {
-		return priv->buffer;
-	}
-
-	priv->current_page = view->current_page;
-
-	if (!priv->buffer) {
-		priv->buffer = gtk_text_buffer_new (NULL);
-	}
-
+	priv->buffer = gtk_text_buffer_new (NULL);
 	retval = ev_page_cache_get_text (page_cache, view->current_page);
 	if (retval)
 		gtk_text_buffer_set_text (priv->buffer, retval, -1);
@@ -306,6 +312,7 @@ static gint
 ev_view_accessible_get_caret_offset (AtkText *text)
 {
 	GtkWidget *widget;
+	EvView *view;
 	GtkTextBuffer *buffer;
 	GtkTextMark *cursor_mark;
 	GtkTextIter cursor_itr;
@@ -316,6 +323,15 @@ ev_view_accessible_get_caret_offset (AtkText *text)
 		/* State is defunct */
 		return 0;
 
+	view = EV_VIEW (widget);
+
+	if (view->caret_enabled) {
+		if (view->cursor_page == view->current_page)
+			return view->cursor_offset;
+		else
+			return -1;
+	}
+
 	buffer = ev_view_accessible_get_text_buffer (EV_VIEW_ACCESSIBLE (text), EV_VIEW (widget));
 	if (!buffer)
 		return 0;
@@ -349,29 +365,127 @@ ev_view_accessible_set_caret_offset (AtkText *text, gint offset)
 	return TRUE;
 }
 
+static AtkAttributeSet *
+add_attribute (AtkAttributeSet  *attr_set,
+               AtkTextAttribute  attr_type,
+               gchar            *attr_value)
+{
+  AtkAttribute *attr = g_new (AtkAttribute, 1);
+
+  attr->name = g_strdup (atk_text_attribute_get_name (attr_type));
+  attr->value = attr_value;
+
+  return g_slist_prepend (attr_set, attr);
+}
+
+static AtkAttributeSet *
+get_run_attributes (PangoAttrList   *attrs,
+		    const gchar     *text,
+		    gint             offset,
+		    gint            *start_offset,
+		    gint            *end_offset)
+{
+	AtkAttributeSet   *atk_attr_set = NULL;
+	PangoAttrString   *pango_string;
+	PangoAttrInt      *pango_int;
+	PangoAttrColor    *pango_color;
+	PangoAttrIterator *iter;
+	gint               i, start, end;
+	gboolean           has_attrs = FALSE;
+	glong              text_length;
+	gchar             *attr_value;
+
+	text_length = g_utf8_strlen (text, -1);
+	if (offset < 0 || offset >= text_length)
+		return NULL;
+
+	/* Check if there are attributes for the offset,
+	 * and set the attributes range if positive */
+	iter = pango_attr_list_get_iterator (attrs);
+	i = g_utf8_offset_to_pointer (text, offset) - text;
+
+	do {
+		pango_attr_iterator_range (iter, &start, &end);
+		if (i >= start && i < end) {
+			*start_offset = g_utf8_pointer_to_offset (text, text + start);
+			if (end == G_MAXINT) /* Last iterator */
+				end = text_length;
+			*end_offset = g_utf8_pointer_to_offset (text, text + end);
+			 has_attrs = TRUE;
+		}
+	} while (!has_attrs && pango_attr_iterator_next (iter));
+
+	if (!has_attrs) {
+		pango_attr_iterator_destroy (iter);
+		return NULL;
+	}
+
+	/* Create the AtkAttributeSet from the Pango attributes */
+	pango_string = (PangoAttrString *) pango_attr_iterator_get (iter, PANGO_ATTR_FAMILY);
+	if (pango_string) {
+		attr_value = g_strdup (pango_string->value);
+		atk_attr_set = add_attribute (atk_attr_set, ATK_TEXT_ATTR_FAMILY_NAME, attr_value);
+	}
+
+	pango_int = (PangoAttrInt *) pango_attr_iterator_get (iter, PANGO_ATTR_SIZE);
+	if (pango_int) {
+		attr_value = g_strdup_printf ("%i", pango_int->value / PANGO_SCALE);
+		atk_attr_set = add_attribute (atk_attr_set, ATK_TEXT_ATTR_SIZE, attr_value);
+	}
+
+	pango_int = (PangoAttrInt *) pango_attr_iterator_get (iter, PANGO_ATTR_UNDERLINE);
+	if (pango_int) {
+		atk_attr_set = add_attribute (atk_attr_set,
+					      ATK_TEXT_ATTR_UNDERLINE,
+					      g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_UNDERLINE,
+										      pango_int->value)));
+	}
+
+	pango_color = (PangoAttrColor *) pango_attr_iterator_get (iter, PANGO_ATTR_FOREGROUND);
+	if (pango_color) {
+		attr_value = g_strdup_printf ("%u,%u,%u",
+					      pango_color->color.red,
+					      pango_color->color.green,
+					      pango_color->color.blue);
+		atk_attr_set = add_attribute (atk_attr_set, ATK_TEXT_ATTR_FG_COLOR, attr_value);
+	}
+
+	pango_attr_iterator_destroy (iter);
+
+	return atk_attr_set;
+}
+
 static AtkAttributeSet*
 ev_view_accessible_get_run_attributes (AtkText *text,
-				       gint    offset,
+				       gint     offset,
 				       gint    *start_offset,
 				       gint    *end_offset)
 {
-	GtkWidget *widget;
-	GtkTextBuffer *buffer;
-	AtkAttributeSet *retval;
+	EvView        *view;
+	GtkWidget     *widget;
+	PangoAttrList *attrs;
+	const gchar   *page_text;
+
+	if (offset < 0)
+		return NULL;
 
 	widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
-	if (widget == NULL)
-		/* State is defunct */
+	if (!widget)
 		return NULL;
 
-	buffer = ev_view_accessible_get_text_buffer (EV_VIEW_ACCESSIBLE (text), EV_VIEW (widget));
-	if (!buffer)
+	view = EV_VIEW (widget);
+	if (!view->page_cache)
 		return NULL;
 
-	retval = gail_misc_buffer_get_run_attributes (buffer, offset,
-	                                              start_offset, end_offset);
+	page_text = ev_page_cache_get_text (view->page_cache, view->current_page);
+	if (!page_text)
+		return NULL;
 
-	return retval;
+	attrs = ev_page_cache_get_text_attrs (view->page_cache, view->current_page);
+	if (!attrs)
+		return NULL;
+
+	return get_run_attributes (attrs, page_text, offset, start_offset, end_offset);
 }
 
 static AtkAttributeSet*
@@ -504,27 +618,54 @@ static gint
 ev_view_accessible_get_n_selections (AtkText *text)
 {
 	GtkWidget *widget;
-	GtkTextBuffer *buffer;
-	GtkTextIter start, end;
-	gint select_start, select_end;
+	EvView *view;
 
 	widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
 	if (widget == NULL)
 		/* State is defunct */
 		return -1;
 
-	buffer = ev_view_accessible_get_text_buffer (EV_VIEW_ACCESSIBLE (text), EV_VIEW (widget));
-	if (!buffer)
-		return -1;
-
-	gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
-	select_start = gtk_text_iter_get_offset (&start);
-	select_end = gtk_text_iter_get_offset (&end);
-
-	if (select_start != select_end)
-		return 1;
-	else
+	view = EV_VIEW (widget);
+	if (!EV_IS_SELECTION (view->document) || !view->selection_info.selections)
 		return 0;
+
+	return 1;
+}
+
+static gboolean
+get_selection_bounds (EvView          *view,
+		      EvViewSelection *selection,
+		      gint            *start_offset,
+		      gint            *end_offset)
+{
+	cairo_rectangle_int_t rect;
+	gint start, end;
+
+	if (!selection->covered_region || cairo_region_is_empty (selection->covered_region))
+		return FALSE;
+
+	cairo_region_get_rectangle (selection->covered_region, 0, &rect);
+	start = _ev_view_get_caret_cursor_offset_at_doc_point (view,
+							       selection->page,
+							       rect.x / view->scale,
+							       (rect.y + (rect.height / 2)) / view->scale);
+	if (start == -1)
+		return FALSE;
+
+	cairo_region_get_rectangle (selection->covered_region,
+				    cairo_region_num_rectangles (selection->covered_region) - 1,
+				    &rect);
+	end = _ev_view_get_caret_cursor_offset_at_doc_point (view,
+							     selection->page,
+							     (rect.x + rect.width) / view->scale,
+							     (rect.y + (rect.height / 2)) / view->scale);
+	if (end == -1)
+		return FALSE;
+
+	*start_offset = start;
+	*end_offset = end;
+
+	return TRUE;
 }
 
 static gchar *
@@ -534,9 +675,13 @@ ev_view_accessible_get_selection (AtkText *text,
 				  gint    *end_pos)
 {
 	GtkWidget *widget;
-	GtkTextBuffer *buffer;
-	GtkTextIter start, end;
-	gchar *retval = NULL;
+	EvView    *view;
+	gchar *selected_text = NULL;
+	gchar *normalized_text = NULL;
+	GList *l;
+
+	*start_pos = -1;
+	*end_pos = -1;
 
 	widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
 	if (widget == NULL)
@@ -546,18 +691,46 @@ ev_view_accessible_get_selection (AtkText *text,
 	if (selection_num != 0)
 		return NULL;
 
-	buffer = ev_view_accessible_get_text_buffer (EV_VIEW_ACCESSIBLE (text), EV_VIEW (widget));
-	if (!buffer)
+	view = EV_VIEW (widget);
+	if (!EV_IS_SELECTION (view->document) || !view->selection_info.selections)
 		return NULL;
 
-	gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
-	*start_pos = gtk_text_iter_get_offset (&start);
-	*end_pos = gtk_text_iter_get_offset (&end);
 
-	if (*start_pos != *end_pos)
-		retval = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+	for (l = view->selection_info.selections; l != NULL; l = l->next) {
+		EvViewSelection *selection = (EvViewSelection *)l->data;
+		gint start, end;
 
-	return retval;
+		if (selection->page != view->current_page)
+			continue;
+
+		if (get_selection_bounds (view, selection, &start, &end) && start != end) {
+			EvPage *page;
+
+			page = ev_document_get_page (view->document, selection->page);
+
+			ev_document_doc_mutex_lock ();
+			selected_text = ev_selection_get_selected_text (EV_SELECTION (view->document),
+									page,
+									selection->style,
+									&(selection->rect));
+
+			ev_document_doc_mutex_unlock ();
+
+			g_object_unref (page);
+
+			*start_pos = start;
+			*end_pos = end;
+		}
+
+		break;
+	}
+
+	if (selected_text) {
+		normalized_text = g_utf8_normalize (selected_text, -1, G_NORMALIZE_NFKC);
+		g_free (selected_text);
+	}
+
+	return normalized_text;
 }
 
 static gboolean
@@ -803,13 +976,9 @@ ev_view_accessible_get_links (EvViewAccessible *accessible,
 {
 	EvViewAccessiblePrivate* priv = accessible->priv;
 
-	if (view->current_page == priv->current_page && priv->links)
+	if (priv->links)
 		return priv->links;
 
-	priv->current_page = view->current_page;
-
-	if (priv->links)
-		g_hash_table_destroy (priv->links);
 	priv->links = g_hash_table_new_full (g_direct_hash,
 					     g_direct_equal,
 					     NULL,
@@ -829,6 +998,9 @@ ev_view_accessible_get_link (AtkHypertext *hypertext,
 	EvMapping        *mapping;
 	EvLinkAccessible *atk_link;
 
+	if (link_index < 0)
+		return NULL;
+
 	widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (hypertext));
 	if (widget == NULL)
 		/* State is defunct */
@@ -849,6 +1021,9 @@ ev_view_accessible_get_link (AtkHypertext *hypertext,
 		return NULL;
 
 	n_links = ev_mapping_list_length (link_mapping);
+	if (link_index > n_links - 1)
+		return NULL;
+
 	mapping = ev_mapping_list_nth (link_mapping, n_links - link_index - 1);
 	atk_link = ev_link_accessible_new (EV_VIEW_ACCESSIBLE (hypertext),
 					   EV_LINK (mapping->data),
@@ -924,10 +1099,52 @@ ev_view_accessible_selection_changed (EvView *view,
 	g_signal_emit_by_name (accessible, "text-selection-changed");
 }
 
+static void
+page_changed_cb (EvDocumentModel  *model,
+		 gint              old_page,
+		 gint              new_page,
+		 EvViewAccessible *accessible)
+{
+	clear_cache (accessible);
+}
+
+static void
+document_changed_cb (EvDocumentModel  *model,
+		     GParamSpec       *pspec,
+		     EvViewAccessible *accessible)
+{
+	clear_cache (accessible);
+}
+
+void
+ev_view_accessible_set_model (EvViewAccessible *accessible,
+			      EvDocumentModel  *model)
+{
+	EvViewAccessiblePrivate* priv = accessible->priv;
+
+	if (priv->model == model)
+		return;
+
+	if (priv->model) {
+		g_signal_handlers_disconnect_by_data (priv->model, accessible);
+		g_object_unref (priv->model);
+	}
+
+	priv->model = g_object_ref (model);
+
+	g_signal_connect (priv->model, "page-changed",
+			  G_CALLBACK (page_changed_cb),
+			  accessible);
+	g_signal_connect (priv->model, "notify::document",
+			  G_CALLBACK (document_changed_cb),
+			  accessible);
+}
+
 AtkObject *
 ev_view_accessible_new (GtkWidget *widget)
 {
 	AtkObject *accessible;
+	EvView    *view;
 
 	g_return_val_if_fail (EV_IS_VIEW (widget), NULL);
 
@@ -941,6 +1158,11 @@ ev_view_accessible_new (GtkWidget *widget)
 			  G_CALLBACK (ev_view_accessible_selection_changed),
 			  accessible);
 
+	view = EV_VIEW (widget);
+	if (view->model)
+		ev_view_accessible_set_model (EV_VIEW_ACCESSIBLE (accessible),
+					      view->model);
+
 	return accessible;
 }
 
